from django.urls import re_path
from pretix.multidomain import event_url

from .views import EventCssJsSettingsView, CustomCssView, CustomJsView

event_patterns = [
    event_url(r'^event_css_js/custom\.css$', CustomCssView.as_view(), name='custom.css'),
    event_url(r'^event_css_js/custom\.js$', CustomJsView.as_view(), name='custom.js'),
]

urlpatterns = [
    re_path(
        r'^control/event/(?P<organizer>[^/]+)/(?P<event>[^/]+)/event_css_js/settings$',
        EventCssJsSettingsView.as_view(),
        name='settings',
    ),
]



from django import forms
from django.utils.translation import gettext_lazy as _
from pretix.base.forms import SettingsForm


class EventCssJsSettingsForm(SettingsForm):
    event_css_js_css = forms.CharField(
        widget=forms.Textarea(attrs={'rows': 12, 'class': 'monospace'}),
        label=_('Custom CSS'),
        required=False,
        help_text=_('Custom CSS rules that will be loaded on every presale page of this event.'),
    )
    event_css_js_js = forms.CharField(
        widget=forms.Textarea(attrs={'rows': 12, 'class': 'monospace'}),
        label=_('Custom JavaScript'),
        required=False,
        help_text=_(
            'Custom JavaScript that will be loaded at the end of every presale page of this event. '
            'Use with care — incorrect code may break the shop experience for your customers.'
        ),
    )



from django.http import HttpResponse
from django.urls import reverse
from django.views.generic import View
from pretix.control.views.event import EventSettingsFormView, EventSettingsViewMixin

from .forms import EventCssJsSettingsForm

CACHE_MAX_AGE = 3600 * 24 * 365  # 1 year — URL changes on content update


class EventCssJsSettingsView(EventSettingsViewMixin, EventSettingsFormView):
    form_class = EventCssJsSettingsForm
    template_name = 'pretix_event_css_js/control_settings.html'

    def get_success_url(self):
        return reverse('plugins:pretix_event_css_js:settings', kwargs={
            'organizer': self.request.organizer.slug,
            'event': self.request.event.slug,
        })


class CustomCssView(View):
    def get(self, request, *args, **kwargs):
        css = request.event.settings.get('event_css_js_css', default='')
        response = HttpResponse(css or '', content_type='text/css; charset=utf-8')
        if 'v' in request.GET:
            response['Cache-Control'] = 'public, max-age={}'.format(CACHE_MAX_AGE)
        return response


class CustomJsView(View):
    def get(self, request, *args, **kwargs):
        js = request.event.settings.get('event_css_js_js', default='')
        response = HttpResponse(js or '', content_type='application/javascript; charset=utf-8')
        if 'v' in request.GET:
            response['Cache-Control'] = 'public, max-age={}'.format(CACHE_MAX_AGE)
        return response




import hashlib

from django.dispatch import receiver
from django.urls import resolve, reverse
from django.utils.translation import gettext_lazy as _
from pretix.control.signals import nav_event_settings
from pretix.multidomain.urlreverse import eventreverse
from pretix.presale.signals import html_footer, html_head


def _content_hash(content):
    return hashlib.md5(content.encode('utf-8')).hexdigest()[:12]


@receiver(nav_event_settings, dispatch_uid='event_css_js_nav_settings')
def navbar_settings(sender, request, **kwargs):
    url = resolve(request.path_info)
    return [{
        'label': _('Event CSS & JS'),
        'url': reverse('plugins:pretix_event_css_js:settings', kwargs={
            'event': request.event.slug,
            'organizer': request.organizer.slug,
        }),
        'active': url.namespace == 'plugins:pretix_event_css_js' and url.url_name == 'settings',
    }]


@receiver(html_head, dispatch_uid="pretix_event_css_js_html_head")
def r_html_head(sender, request=None, **kwargs):
    css = sender.settings.get('event_css_js_css', default='')
    if css and css.strip():
        url = eventreverse(sender, 'plugins:pretix_event_css_js:custom.css')
        url += '?v=' + _content_hash(css)
        return '<link rel="stylesheet" type="text/css" href="{}">'.format(url)
    return ''


@receiver(html_footer, dispatch_uid="pretix_event_css_js_html_footer")
def html_foot_presale(sender, request=None, **kwargs):
    js = sender.settings.get('event_css_js_js', default='')
    if js and js.strip():
        url = eventreverse(sender, 'plugins:pretix_event_css_js:custom.js')
        url += '?v=' + _content_hash(js)
        return '<script src="{}"></script>'.format(url)
    return ''



from django.utils.translation import gettext_lazy
from . import __version__

try:
    from pretix.base.plugins import PluginConfig
except ImportError:
    raise RuntimeError("Please use pretix 2.7 or above to run this plugin!")


class PluginApp(PluginConfig):
    default = True
    name = "pretix_event_css_js"
    verbose_name = "Event CSS & JS"

    class PretixPluginMeta:
        name = gettext_lazy("Event CSS & JS")
        author = "Nico Knoll"
        description = gettext_lazy("Inject custom CSS and JavaScript into the presale pages of individual events.")
        visible = True
        version = __version__
        category = "CUSTOMIZATION"
        compatibility = "pretix>=2.7.0"

    def ready(self):
        from . import signals  # NOQA

    def uninstalled(self, event):
        event.settings.delete('event_css_js_css')
        event.settings.delete('event_css_js_js')
